from ward import test, raises
import os
from .parser import Issue, parse
import textwrap

for fragment, expected, description_expected in [
    ("", Issue(), False),
    ("a simple test right there", Issue(title="a simple test right there"), False),
    (
        "@me some ~labels to ~organize issues ~bug",
        Issue(
            title="some labels to organize issues",
            labels={"labels", "organize", "bug"},
            assignees={"me"},
        ),
        False,
    ),
    (
        "a %milestone to keep ~track of stuff",
        Issue(
            title="a milestone to keep track of stuff",
            labels={"track"},
            milestone="milestone",
        ),
        False,
    ),
    (
        "A label with a description following it ~now:",
        Issue(title="A label with a description following it", labels={"now"}),
        True,
    ),
    (
        "#.1 An issue with a reference definition",
        Issue(title="An issue with a reference definition", reference=1),
        False,
    ),
]:

    @test(f"parse {fragment!r}")
    def _(
        fragment=fragment, expected=expected, description_expected=description_expected
    ):
        actual, expecting_description = Issue.parse(fragment)
        assert expecting_description == description_expected
        assert actual == expected


for lines, expected in [
    ("", []),
    ("A simple issue", [Issue(title="A simple issue")]),
    ("~label @me", []),
    (
        """
        @me some ~labels to ~organize issues ~bug
        a %milestone to keep ~track of stuff
        """,
        [
            Issue(
                title="some labels to organize issues",
                labels={"labels", "organize", "bug"},
                assignees={"me"},
            ),
            Issue(
                title="a milestone to keep track of stuff",
                labels={"track"},
                milestone="milestone",
            ),
        ],
    ),
    (
        """
        some stuff
        \tinside: not processed
        """,
        [
            Issue(title="some stuff"),
        ],
    ),
    (
        """
        ~common-tag @someone
        \tright there ~other-tag
        \t//A comment

        \t@someone-else right %here
        """,
        [
            Issue(
                title="right there",
                labels={"common-tag", "other-tag"},
                assignees={"someone"},
            ),
            Issue(
                title="right",
                labels={"common-tag"},
                assignees={"someone-else", "someone"},
                milestone="here",
            ),
        ],
    ),
    (
        """An ~issue with a description:
\tThis is the %description of the issue:
\t// This is *not* a comment
\tIt has a 
\t- bullet list

\tAnd
\t\tIndentation
        """,
        [
            Issue(
                title="An issue with a description",
                labels={"issue"},
                description="""This is the %description of the issue:
// This is *not* a comment
It has a
- bullet list
And
\tIndentation
""",
            )
        ],
    ),
    (
        """%milestone_test @me
\t~notsure do this
\t~important do that
""",
        [
            Issue(
                title="do this",
                labels={"notsure"},
                assignees={"me"},
                milestone="milestone_test",
            ),
            Issue(
                title="do that",
                labels={"important"},
                assignees={"me"},
                milestone="milestone_test",
            ),
        ],
    ),
    (
        """
An issue that references another ~blocked:
\tSee #.1

#.1 The other one ^w^
""",
        [
            Issue(
                title="An issue that references another",
                labels={"blocked"},
                description="See #.1\n",
            ),
            Issue(
                title="The other one ^w^",
                reference=1,
            ),
        ],
    ),
]:

    @test(f"parse issues from {textwrap.dedent(lines)!r}")
    def _(lines=lines, expected=expected):
        assert list(parse(lines)) == expected


@test("resolves references")
def _():
    [issue, *_] = list(parse("An issue that references another ~blocked:\n\tSee #.2"))
    assert issue.references == {2}
    issue = issue.resolve_references({2: 1})
    assert issue.description == "See #1\n"

@test("splits #. sigil")
def _():
    assert Issue._word_and_sigil("#.1") == ("#.", "1")

@test("parse issue with missing description fails")
def _():
    with raises(ValueError) as exception:
        list(parse("An ~issue with a description:\nNo description here"))
    assert (
        str(exception.raised)
        == f"Expected a description after 'An ~issue with a description:'"
    )



import os
from urllib.parse import urlparse
from ward import fixture, test
from issurge.main import run
from pathlib import Path
from unittest.mock import Mock
from issurge.parser import Issue, subprocess

from issurge.utils import debugging, dry_running

class MockedSubprocessOutput:
    def __init__(self, stdout: str, stderr: str):
        self.stdout = stdout.encode('utf-8')
        self.stderr = stderr.encode('utf-8')

@fixture
def setup():
    Path("test_empty_issues").write_text("")
    Path("test_some_issues").write_text(
        """~common @common %common
\tAn issue to submit
Another ~issue to submit @me"""
    )
    subprocess.run = Mock(
        return_value=MockedSubprocessOutput("https://github.com/gwennlbh/gh-api-playground/issues/5\n", "Some unrelated stuff haha")
    )
    Issue._get_remote_url = Mock(
        return_value=urlparse("https://github.com/ewen-lbh/gh-api-playground")
    )
    yield
    Path("test_empty_issues").unlink()
    Path("test_some_issues").unlink()
    del os.environ["ISSURGE_DEBUG"]
    del os.environ["ISSURGE_DRY_RUN"]


@fixture
def default_opts():
    yield {
        "<submitter-args>": [],
        "<file>": "test_empty_issues",
        "<words>": [],
        "new": False,
        "--dry-run": False,
        "--debug": False,
    }


@test("dry run is set when --dry-run is passed")
def _(_=setup, opts=default_opts):
    run(opts=opts | {"--dry-run": True})
    assert dry_running()
    assert not debugging()


@test("debug is set when --debug is passed")
def _(_=setup, opts=default_opts):
    run(opts=opts | {"--debug": True})
    assert debugging()
    assert not dry_running()


@test("dry run and debug are not set by default")
def _(_=setup, opts=default_opts):
    run(opts=opts)
    assert not dry_running()
    assert not debugging()


@test("both dry run and debug are set when both are passed")
def _(_=setup, opts=default_opts):
    run(opts=opts | {"--dry-run": True, "--debug": True})
    assert dry_running()
    assert debugging()


@test("issues are submitted when --dry-run is not passed, with github provider")
def _(_=setup, opts=default_opts):
    run(opts=opts | {"<file>": "test_some_issues"})
    assert [call.args[0] for call in subprocess.run.mock_calls] == [
        [
            "gh",
            "issue",
            "new",
            "-t",
            "An issue to submit",
            "-b",
            "",
            "-a",
            "common",
            "-l",
            "common",
            "-m",
            "common",
        ],
        [
            "gh",
            "issue",
            "new",
            "-t",
            "Another issue to submit",
            "-b",
            "",
            "-a",
            "@me",
            "-l",
            "issue",
        ],
    ]


@test("issues are submitted when --dry-run is not passed, with gitlab provider")
def _(_=setup, opts=default_opts):
    Issue._get_remote_url = Mock(
        return_value=urlparse("https://gitlab.com/ewen-lbh/gh-api-playground")
    )
    run(opts=opts | {"<file>": "test_some_issues"})
    assert [call.args[0] for call in subprocess.run.mock_calls] == [
        [
            "glab",
            "issue",
            "new",
            "-t",
            "An issue to submit",
            "-d",
            "",
            "-a",
            "common",
            "-l",
            "common",
            "-m",
            "common",
        ],
        [
            "glab",
            "issue",
            "new",
            "-t",
            "Another issue to submit",
            "-d",
            "",
            "-a",
            "@me",
            "-l",
            "issue",
        ],
    ]


@test("issues are not submitted when --dry-run is passed")
def _(_=setup, opts=default_opts):
    run(opts=opts | {"<file>": "test_some_issues", "--dry-run": True})
    assert len(subprocess.run.mock_calls) == 0


@test("issues are not submitted when --dry-run is passed, in interactive mode")
def _(_=setup, opts=default_opts):
    run(
        opts=opts
        | {
            "new": True,
            "--dry-run": True,
            "<words>": ["testing", "~this", "issue", "@me"],
        }
    )
    assert len(subprocess.run.mock_calls) == 0


@test(
    "issues are submitted when --dry-run is not passed, in interactive mode, github provider"
)
def _(_=setup, opts=default_opts):
    run(
        opts=opts
        | {
            "new": True,
            "<words>": ["testing", "~this", "issue", "@me"],
        }
    )
    assert [call.args[0] for call in subprocess.run.mock_calls] == [
        [
            "gh",
            "issue",
            "new",
            "-t",
            "testing this issue",
            "-b",
            "",
            "-a",
            "@me",
            "-l",
            "this",
        ],
    ]


@test(
    "issues are submitted when --dry-run is not passed, in interactive mode, gitlab provider"
)
def _(_=setup, opts=default_opts):
    Issue._get_remote_url = Mock(
        return_value=urlparse("https://gitlab.com/ewen-lbh/gh-api-playground")
    )
    run(
        opts=opts
        | {
            "new": True,
            "<words>": ["testing", "~this", "issue", "@me"],
        }
    )
    assert [call.args[0] for call in subprocess.run.mock_calls] == [
        [
            "glab",
            "issue",
            "new",
            "-t",
            "testing this issue",
            "-d",
            "",
            "-a",
            "@me",
            "-l",
            "this",
        ],
    ]



import os, io, sys
from unittest.mock import Mock
from ward import test
from issurge.utils import debug, debugging, dry_running
import issurge.utils


@test("debugging is false by default")
def _():
    assert not debugging()


@test("debugging is true when ISSURGE_DEBUG is set")
def _():
    os.environ["ISSURGE_DEBUG"] = "1"
    assert debugging()
    del os.environ["ISSURGE_DEBUG"]


@test("dry_running is false by default")
def _():
    assert not dry_running()


@test("dry_running is true when ISSURGE_DRY_RUN is set")
def _():
    os.environ["ISSURGE_DRY_RUN"] = "1"
    assert dry_running()
    del os.environ["ISSURGE_DRY_RUN"]


@test("debug only prints when debugging is true")
def _():
    issurge.utils.print = Mock()
    os.environ["ISSURGE_DEBUG"] = "1"
    debug("debug")
    assert len(issurge.utils.print.mock_calls) == 1
    assert issurge.utils.print.mock_calls[0].args[0] == "debug"



from unittest.mock import Mock

from ward import test

from issurge import interactive
from issurge.parser import Issue

for words, expected in [
    ("", Issue()),
    ("a simple test right there", Issue(title="a simple test right there")),
    (
        "@me some ~labels to ~organize issues ~bug",
        Issue(
            title="some labels to organize issues",
            labels={"labels", "organize", "bug"},
            assignees={"me"},
        ),
    ),
    (
        "a %milestone to keep ~track of stuff",
        Issue(
            title="a milestone to keep track of stuff",
            labels={"track"},
            milestone="milestone",
        ),
    ),
]:

    @test("create issue from words {words!r}")
    def _(words=words, expected=expected):
        actual = interactive.create_issue(words)
        assert actual == expected


@test("create issue from words with description")
def _():
    global called_count
    called_count = 0

    def mocked_input(prompt):
        global called_count
        called_count += 1
        return {
            1: "A description",
            2: "",
            3: "Another line",
            4: "With more content",
        }.get(called_count, "")

    interactive.input = Mock(wraps=mocked_input)
    actual = interactive.create_issue("A label with a description following it ~now:")
    expected = Issue(
        title="A label with a description following it",
        labels={"now"},
        description="A description\n\nAnother line\nWith more content",
    )
    assert actual == expected
    assert len(interactive.input.mock_calls) == 6



#!/usr/bin/env python
"""
Usage:
    issurge [options] new <words>...
    issurge [options] <file> [--] [<submitter-args>...]
    issurge --help

issurge new <words>... acts like echo <words>... | issurge /dev/stdin, but also asks for a description if the issue ends with `:'.

<submitter-args> contains arguments that will be passed as-is to the end of all `glab' commands

Options:
    --dry-run   Don't actually post the issues
    --debug     Print debug information
"""

import os
from pathlib import Path

from docopt import docopt
from rich import print

from issurge.parser import parse
from issurge.utils import debug
from issurge import interactive


def run(opts=None):
    opts = opts or docopt(__doc__)
    os.environ["ISSURGE_DEBUG"] = "1" if opts["--debug"] else ""
    os.environ["ISSURGE_DRY_RUN"] = "1" if opts["--dry-run"] else ""

    debug(f"Running with options: {opts}")
    if opts["new"]:
        issue = interactive.create_issue(" ".join(opts["<words>"]))
        debug(f"Submitting {issue.display()}")
        number = issue.submit(opts["<submitter-args>"])
        print(f"Created issue #{number}")
    else:
        print("Submitting issues...")
        references_resolutions: dict[int, int] = {}
        for issue in parse(Path(opts["<file>"]).read_text()):
            issue = issue.resolve_references(references_resolutions, strict=True)
            number = issue.submit(opts["<submitter-args>"])
            print(f"Created issue #{number}")
            if issue.reference and number:
                references_resolutions[issue.reference] = number
